<?php
use Braintree\PaymentInstrumentType;

/**
 * Return the merchant account id configured for the WooCommerce currency.
 * If there is no merchant account, return an empty string.
 */
function bwc_get_merchant_account( $currency = '' )
{
	$currency = empty( $currency ) ? get_woocommerce_currency() : $currency;
	$key = sprintf( 'woocommerce_braintree_%s_merchant_account_id', bt_manager()->get_environment() );
	$merchant_accounts = bt_manager()->get_option( $key );
	
	if ( ! empty( $merchant_accounts ) ) {
		
		return isset( $merchant_accounts [ $currency ] ) ? $merchant_accounts [ $currency ] : '';
	} else {
		return '';
	}
}

/**
 * Return true if dynamic descriptors have been enabled.
 *
 * @return boolean
 */
function bwc_is_descriptors_enabled()
{
	return bt_manager()->is_active( 'dynamic_descriptors' );
}

/**
 * Return true if custom forms are enabled.
 *
 * @return boolean
 */
function bwc_is_custom_form()
{
	return bt_manager()->get_option( 'checkout_form' ) === 'custom_form';
}

/**
 * Return true if 3DS is active.
 * If WC subscriptions is active and
 * the request is for a payment method change, this function will return false
 * regardless of
 * the 3DS setting.
 *
 * @return boolean
 */
function bwc_is_3ds_active()
{
	if ( ! bwc_is_checkout() ) {
		return false; // only allow 3DS check on checkout page since 3DS requires an amount.
	}
	return bt_manager()->is_active( '3ds_enabled' );
}

/**
 * Return true if Apple Pay is enabled.
 *
 * @return boolean
 */
function bwc_is_applepay_enabled()
{
	return bt_manager()->is_active( 'enable_applepay' );
}

/**
 * Return true of the order contains a transaction Id.
 *
 * @param WC_Order $order        	
 */
function bwc_can_refund_order( $order )
{
	$id = $order->get_transaction_id();
	return ! empty( $id );
}

/**
 * Return an array of custom form fields used in the custom payment form.
 *
 * @return mixed
 */
function bwc_get_custom_form_fields()
{
	return apply_filters( 'braintree_woocommerce_custom_form_fields', array (
			'number' => array (
					'selector' => '#bfwc-card-number', 
					'placeholder' => bt_manager()->get_option( 'card_number_placeholder' ) 
			), 
			'cvv' => array (
					'selector' => '#bfwc-cvv', 
					'placeholder' => bt_manager()->get_option( 'card_cvv_placeholder' ) 
			), 
			'postalCode' => array (
					'selector' => '#bfwc-postal-code', 
					'placeholder' => bt_manager()->get_option( 'card_postal_placeholder' ) 
			), 
			'expirationDate' => array (
					'selector' => '#bfwc-expiration-date', 
					'placeholder' => bt_manager()->get_option( 'card_expiration_date_placeholder' ) 
			), 
			'expirationMonth' => array (
					'selector' => '#bfwc-expiration-month', 
					'placeholder' => bt_manager()->get_option( 'card_expiration_month_placeholder' ) 
			), 
			'expirationYear' => array (
					'selector' => '#bfwc-expiration-year', 
					'placeholder' => bt_manager()->get_option( 'card_expiration_year_placeholder' ) 
			) 
	) );
}

/**
 * Return the configured custom form.
 * <strong>Example</strong>
 * array('html' => 'custom-forms/bootstrap-form.php',
 * 'css'=>'https:'//example.com/styles/mycss.css')
 *
 * @return array
 */
function bwc_get_custom_form()
{
	$form = bt_manager()->get_option( 'custom_form_design' );
	$forms = bwc_get_custom_forms();
	return $forms [ $form ];
}

/**
 * Return an array of custom form.
 *
 * @return array
 */
function bwc_get_custom_forms()
{
	return include bt_manager()->plugin_include_path() . 'braintree-custom-forms.php';
}

/**
 * Return the html for the 3DS modal html.
 */
function bwc_get_3ds_modal_html()
{
	ob_start();
	bwc_get_template( 'custom-forms/3ds-modal.php' );
	return ob_get_clean();
}

/**
 * Return true if the payment method icons should be displayed on the outside of
 * the gateway html.
 *
 * @return boolean
 */
function bwc_payment_icons_outside()
{
	return bt_manager()->get_option( 'payment_method_location' ) === 'outside';
}

/**
 * Return true if only PayPal is the chosen payment gateway.
 *
 * @return boolean
 */
function bwc_is_paypal_only()
{
	return bt_manager()->is_active( 'paypal_only' );
}

/**
 * Return true if PayPal has been enabled as a payment gateway.
 * Custom forms
 * must be enabled as well in order for this function to return true.
 *
 * @return boolean
 */
function bwc_is_paypal_enabled()
{
	if ( bwc_is_paypal_only() ) {
		return true;
	} else {
		return bt_manager()->is_active( 'enable_paypal' ) && bwc_is_custom_form();
	}
}

/**
 * Return the button that has been selected for use on the frontend.
 *
 * @return array
 */
function bwc_get_paypal_button()
{
	$button = bt_manager()->get_option( 'paypal_button_design' );
	$buttons = braintree_get_paypal_buttons();
	return $buttons [ $button ];
}

/**
 * Return the gateway id for the given gateway.
 *
 * @param string $gateway
 *        	braintree | paypal
 * @return string
 */
function bwc_get_gateway_id( $gateway )
{
	switch( $gateway) {
		case 'braintree' :
			return WC_Braintree_Payment_Gateway::ID;
			break;
		case 'paypal' :
			return WC_PayPal_Payment_Gateway::ID;
			break;
		case 'applepay' :
			return WC_Applepay_Payment_Gateway::ID;
			break;
	}
}

/**
 * Return the html used for paypal methods.
 *
 * @return string
 */
function bwc_get_paypal_html()
{
	ob_start();
	bwc_get_template( 'paypal/paypal-vaulted.php' );
	return ob_get_clean();
}

/**
 * Return true if the PayPal checkout flow is 'checkout.'
 * If WooCommerce Subscriptions is active, then check if the cart contains
 * subscriptions.
 * If there are subscriptions in the cart, then 'vault' must be active to ensure
 * PayPal payment methods are saved. If the page is the add payment page, then
 * false is returned as 'vault' flow
 * is needed for adding PayPal payment methods.
 *
 * @return string 'checkout' | 'vault'
 */
function bwc_paypal_checkout_flow()
{
	$flow = bt_manager()->get_option( 'paypal_checkout_flow' );
	
	if ( class_exists( 'WC_Subscriptions_Cart' ) ) {
		if ( WC_Subscriptions_Cart::cart_contains_subscription() ) {
			return false;
		}
	}
	if ( bfwcs_cart_contains_subscriptions() ) {
		return false;
	}
	if ( is_add_payment_method_page() ) {
		return false;
	}
	
	return $flow === 'checkout';
}

/**
 * Return true if the gateway is configured to reject duplicate payment methods.
 *
 * @return boolean
 */
function bwc_fail_on_duplicate()
{
	return bt_manager()->is_active( 'fail_on_duplicate' );
}

/**
 * Return true if advanced fraud tools has been enabled.
 *
 * @return boolean
 */
function bwc_is_advanced_fraud_tools()
{
	return bt_manager()->is_active( 'advanced_fraud_enabled' );
}

/**
 * Include the given template.
 * The template is first checked in the theme's path.
 *
 * @param string $template        	
 * @param array $args        	
 */
function bwc_get_template( $template, $args = array() )
{
	extract( $args );
	
	$located = bwc_locate_template( $template );
	
	if ( ! file_exists( $located ) ) {
		_doing_it_wrong( __FUNCTION__, sprintf( 'File %s does not exist.', $template ), bt_manager()->version );
		return;
	}
	
	// Allow other plugins to replace the file if desired.
	$located = apply_filters( "braintree_woocommerce_template_{$template}", $located, $args );
	
	include $located;
}

/**
 * Loate the template file in the theme and if not in the theme then in the
 * plugin.
 *
 * @param unknown $template_names        	
 */
function bwc_locate_template( $template_name )
{
	$default_path = bt_manager()->plugin_path() . 'templates/';
	
	$located = locate_template( trailingslashit( bt_manager()->template_path() ) . $template_name );
	
	if ( ! $located ) {
		$located = $default_path . $template_name;
	}
	return $located;
}

function bwc_is_checkout()
{
	return is_checkout() && ! bwcs_is_change_payment_method();
}

function bwc_get_user_payment_methods( $user_id )
{
	$methods = braintree_get_user_payment_methods( $user_id );
	// If PayPal is enabled, only return non PayPal payment methods.
	if ( bwc_is_paypal_enabled() ) {
		foreach ( $methods as $key => $method )
			if ( $method [ 'type' ] === PaymentInstrumentType::PAYPAL_ACCOUNT ) {
				unset( $methods [ $key ] );
			}
	}
	if ( bwc_is_applepay_enabled() ) {
		foreach ( $methods as $key => $method )
			if ( $method [ 'type' ] === PaymentInstrumentType::APPLE_PAY_CARD ) {
				unset( $methods [ $key ] );
			}
	}
	return $methods;
}

function bwc_get_user_paypal_payment_methods( $user_id )
{
	$methods = braintree_get_user_payment_methods( $user_id );
	// If PayPal is enabled, only return non PayPal payment methods.
	if ( bwc_is_paypal_enabled() ) {
		foreach ( $methods as $key => $method )
			if ( $method [ 'type' ] !== PaymentInstrumentType::PAYPAL_ACCOUNT ) {
				unset( $methods [ $key ] );
			}
	}
	return $methods;
}

function bwc_get_user_applepay_payment_methods( $user_id )
{
	$methods = braintree_get_user_payment_methods( $user_id );
	// If PayPal is enabled, only return non PayPal payment methods.
	if ( bwc_is_applepay_enabled() ) {
		foreach ( $methods as $key => $method )
			if ( $method [ 'type' ] !== PaymentInstrumentType::APPLE_PAY_CARD ) {
				unset( $methods [ $key ] );
			}
	}
	return $methods;
}

/**
 * Return an array of saved payment methods.
 * Most useful if called
 * when the payment-methods page is being rendered.
 *
 * @param unknown $methods        	
 * @return mixed
 */
function bwc_saved_payment_methods_list( $saved_methods, $methods, $gateway_id )
{
	foreach ( $methods as $token => $method ) {
		$data = array (
				'method' => array (), 
				'expires' => $method [ 'type' ] === PaymentInstrumentType::PAYPAL_ACCOUNT ? __( 'N/A', 'braintree' ) : sprintf( '%s / %s', $method [ 'exp_month' ], $method [ 'exp_year' ] ), 
				
				'actions' => array (
						'delete' => array (
								'name' => __( 'Delete', 'braintree' ), 
								'url' => wp_nonce_url( add_query_arg( "{$gateway_id}_delete_method", $token, wc_get_endpoint_url( 'payment-methods', '', get_permalink( wc_get_page_id( 'myaccount' ) ) ) ), 'delete-payment-method' ) 
						) 
				) 
		);
		switch( $method [ 'type' ]) {
			case PaymentInstrumentType::CREDIT_CARD :
				$data [ 'method' ] [ 'last4' ] = $method [ 'last4' ];
				$data [ 'method' ] [ 'brand' ] = $method [ 'card_type' ];
				break;
			case PaymentInstrumentType::PAYPAL_ACCOUNT :
				$data [ 'method' ] [ 'brand' ] = sprintf( __( 'PayPal - %s', 'braintree' ), $method [ 'email' ] );
				break;
			case PaymentInstrumentType::APPLE_PAY_CARD :
				$last4 = preg_match( '/[\d]+/', $method [ 'payment_instrument_name' ], $matches ) ? $matches [ 0 ] : '';
				$data [ 'method' ] [ 'brand' ] = preg_match( '/[a-z]+/i', $method [ 'payment_instrument_name' ], $matches ) ? sprintf( __( 'Apple Pay %s', 'braintree' ), $matches [ 0 ] ) : $method [ 'card_type' ];
				$data [ 'method' ] [ 'last4' ] = $last4;
				break;
		}
		
		if ( $method [ 'default' ] ) {
			$data [ 'is_default' ] = 'true';
		}
		$saved_methods [ $gateway_id ] [] = $data;
	}
	
	return apply_filters( 'braintree_saved_payment_methods_list', $saved_methods );
}

/**
 * Return true if dynamic card display is active.
 *
 * @return boolean
 */
function bwc_is_dynamic_card_display()
{
	return bt_manager()->is_active( 'dynamic_card_display' );
}

/**
 * Echo an input field for the payment method token.
 *
 * @param string $token        	
 */
function bwc_payment_method_token_field( $id, $token = '' )
{
	$field = '<input type="hidden" class="bfwc-payment-method-token" id="' . $id . '" name="' . $id . '" value="' . $token . '"/>';
	echo $field;
}

/**
 * Return an array of Apple Pay buttons.
 *
 * @return array
 */
function bwc_get_applepay_buttons()
{
	$buttons = require bt_manager()->plugin_include_path() . 'braintree-applepay-buttons.php';
	return $buttons;
}

/**
 * Return the button that has been configured for Apple Pay.
 *
 * @return array
 */
function bwc_get_applepay_button()
{
	$button_id = bt_manager()->get_option( 'applepay_button' );
	$buttons = bwc_get_applepay_buttons();
	return $buttons [ $button_id ];
}

/**
 * Return an array of braintree WC gateways.
 *
 * @return array
 */
function bwc_get_payment_gateways()
{
	$gateways = apply_filters( 'bwc_add_payment_gateways', array () );
	$wc_gateways = array ();
	foreach ( $gateways as $class ) {
		$wc_gateways [] = $class::ID;
	}
	return $wc_gateways;
}

function bwc_get_default_method( $methods = array() )
{
	$default_method = null;
	
	foreach ( $methods as $method ) {
		if ( $method [ 'default' ] ) {
			return $method;
		} else {
			if ( is_null( $default_method ) || $method [ 'created_at' ] > $default_method [ 'created_at' ] ) {
				$default_method = $method;
			}
		}
	}
	return $default_method;
}

/**
 * Return a class name for the payment method.
 *
 * @param array $method        	
 * @return mixed|string
 */
function bwc_get_payment_method_class( $method )
{
	switch( $method [ 'type' ]) {
		case PaymentInstrumentType::CREDIT_CARD :
		case PaymentInstrumentType::PAYPAL_ACCOUNT :
			return str_replace( ' ', '', $method [ 'method_type' ] );
			break;
		case PaymentInstrumentType::APPLE_PAY_CARD :
			return preg_match( '/[a-z]+/i', $method [ 'payment_instrument_name' ], $matches ) ? $matches [ 0 ] : $method [ 'method_type' ];
			break;
	}
}

function bwc_get_save_method_template()
{
	bwc_get_template( 'checkout/save-method.php' );
}

/**
 * Return true if the save payment method checkbox should be displayed.
 */
function bwc_display_save_payment_method()
{
	if ( bt_manager()->is_active( 'save_payment_methods' ) && is_user_logged_in() ) {
		
		// must be checkout page to display save payment method checkbox.
		if ( bwc_is_checkout() ) {
			if ( class_exists( 'WC_Subscriptions_Cart' ) && WC_Subscriptions_Cart::cart_contains_subscription() ) {
				return false;
			}
			if ( bfwcs_cart_contains_subscriptions() ) {
				return false;
			} else {
				return true;
			}
		} else {
			return false;
		}
	} else {
		return false;
	}
}

function bwc_saved_payment_method_style()
{
	return bt_manager()->get_option( 'saved_payment_methods_style' );
}