<?php

/**
 * Functions that pertains to the Braintree subscription functionality.
 * This includes functions specific to WC Subscriptions and the Braintree Subscriptions.
 */

/**
 * Return true if Braintree subscriptions is active.
 *
 * @return boolean
 */
function bwcs_subscriptions_active()
{
	return bt_manager()->is_woocommerce_subscriptions_active();
}

/**
 * Return true if subscriptions are to be created using Braintree's recurring
 * plan options.
 *
 * @return boolean
 */
function bwcs_braintree_subscriptions_active()
{
	return bt_manager()->is_active( 'braintree_subscriptions' );
}

/**
 * Return true if the WC_Order is a Braintree_Subscription.
 *
 * @param WC_Order $order 
 */
function bwcs_order_is_subscription( $order )
{
	return $order->subscription_type === 'braintree';
}

/**
 * Function that determines if there are any subscriptions associated with the
 * payment method.
 * If so, then it returns false. Subscriptions with a status of cancelled are
 * not considered.
 *
 * @param WP_Error $error 
 * @param string $token 
 * @return boolean
 */
function bwcs_can_delete_payment_method( $error, $token )
{
	global $wpdb;
	
	$statuses = wcs_get_subscription_statuses();
	unset( $statuses[ 'wc-cancelled' ] );
	unset( $statuses[ 'wc-pending-cancel' ] );
	
	$in_statuses = implode( "','", array_fill( 0, count( $statuses ), '%s' ) );
	$in_statuses = vsprintf( $in_statuses, array_keys( $statuses ) );
	
	$query = $wpdb->prepare( "SELECT id FROM $wpdb->posts AS posts INNER JOIN $wpdb->postmeta AS postmeta 
            ON posts.ID = postmeta.post_id WHERE posts.post_type = 'shop_subscription'
            AND posts.post_status IN ('{$in_statuses}') AND postmeta.meta_key = '_payment_method_token' AND postmeta.meta_value = %s", $token );
	
	$results = $wpdb->get_results( $query );
	
	if ( $results ) {
		$message = sprintf( _n( 'Your payment method cannot be deleted. Reason: There is %s subscription associated with this payment method.', 'Your payment method cannot be deleted. Reason: There are %s subscriptions associated with this payment method.', count( $results ), 'braintree' ), count( $results ) );
		$error->add( 'method-delete-error', $message );
	}
	return $error;
}

/**
 * Return true if the request is for a subscription payment method change.
 */
function bwcs_is_change_payment_method()
{
	return isset( $_GET[ 'change_payment_method' ] );
}

/**
 * Return true if the request is for when a subscription payment method has been
 * changed.
 */
function bwcs_is_woocommerce_change_payment()
{
	return isset( $_POST[ 'woocommerce_change_payment' ] );
}

/**
 * Return true if the cart item contains a Braintree Subscription.
 *
 * @param array $cart_item 
 * @return boolean
 */
function bwcs_cart_item_contains_subscription( $cart_item )
{
	$product = $cart_item[ 'data' ];
	
	return get_post_meta( $product->id, '_braintree_subscription', true ) === 'yes';
}

/**
 * Returns true if a product is a Braintree Subscription.
 *
 * @param
 * mixed int|WC_Product $product_id
 * @return boolean
 */
function bwcs_product_is_subscription( $product_id )
{
	if ( is_object( $product_id ) ) {
		$product_id = $product_id->id;
	}
	if ( bwcs_subscriptions_active() ) {
		// backwards compatability.
		return WC_Subscriptions_Product::is_subscription( $product_id ) && get_post_meta( $product_id, '_braintree_subscription', true ) === 'yes';
	}
	return get_post_meta( $product_id, '_braintree_subscription', true ) === 'yes';
}

/**
 * Returns true of the order contains a Braintree Subscription.
 *
 * @param
 * mixed int|WC_Order $order
 */
function bwcs_order_contains_subscription( $order )
{
	if ( ! is_object( $order ) ) {
		$order = wc_get_order( $order );
	}
	
	foreach ( $order->get_items() as $k => $item ) {
		
		$product = $order->get_product_from_item( $item );
		if ( $product ) {
			if ( get_post_meta( $product->id, '_braintree_subscription', true ) === 'yes' ) {
				return true;
			}
		}
	
	}
	return false;
}

/**
 * Return the Braintree Subscription plan associated with the WC product.
 *
 * @param
 * mixed int|Object $product_id
 * @return mixed boolean|string - false if plans doesn't exist.
 */
function bwcs_get_plan_from_product( $product )
{
	if ( ! is_object( $product ) ) {
		$product = wc_get_product( $product );
	}
	$meta_key = sprintf( 'braintree_%s_plans', bt_manager()->get_environment() );
	$plans = $product->$meta_key;
	
	$currency = get_woocommerce_currency();
	
	return isset( $plans[ $currency ] ) ? $plans[ $currency ] : false;
}

/**
 * Calculate the WC_Order total.
 * If the order contains subscriptions which start immediately, then their totals
 * should be subtracted from the order.
 *
 * @param WC_Order $order 
 * @param WC_Subscription $subscription 
 */
function bwcs_calculate_order_total( $order, $subscription )
{
	if ( ! $subscription->contains_synced_subscription && ! $subscription->get_date( 'trial_end' ) ) {
		if ( $order->get_total() >= $subscription->get_total() ) {
			
			// $subscription->get_total() returns the recurring total, which inlcudes taxes & shipping. Signup fees are exluded.
			$order->set_total( $order->get_total() - $subscription->get_total() );
		}
	
	}
}

/**
 * Remove the items associated with the subscription from the order.
 *
 * @param WC_Subscription $subscription 
 */
function bwcs_get_cart_item_key( $subscription )
{
	foreach ( WC()->cart->cart_contents as $cart_item_key->$cart_contents ) {
		if ( $subscription->has_product( $cart_contents[ 'product_id' ] ) ) {
			return $cart_item_key;
		}
	}
	return '';
}

/**
 *
 * @param WC_Subscription $subscription 
 */
function bwcs_get_product_from_subscription( $subscription )
{
	foreach ( $subscription->get_items() as $k => $item ) {
		if ( $product = $subscription->get_product_from_item( $item ) ) {
			return $product;
		}
	}
	return false;
}

/**
 * Return true of the subscription object is associated with a Braintree subscription.
 *
 * @param WC_Order $subscription 
 */
function bwcs_is_braintree_subscription( $subscription )
{
	if ( is_object( $subscription ) ) {
		$id = $subscription->id;
	} else {
		$id = $subscription;
	}
	return get_post_meta( $id, '_subscription_type', true ) === 'braintree';
}

/**
 * Return true if products that are the same can be combined into one subscription.
 *
 * @return boolean
 */
function bwcs_can_combine_subscriptions()
{
	return bt_manager()->is_active( 'braintree_subscription_combine_same_products' );
}

/**
 * Return the DateTime object that represents the subscription start date.
 * The DateTime object is created using the next_payment for synchronized subscriptions and the start_date for non trial subscriptions.
 * Date and the timezone of the site is then used to determine the local time. A new DateTime object is then creating using that localtime
 * and the timezone is set to UTC, which does not affect the time.
 *
 * @param WC_Subscription $subscription 
 * @param string $format
 * - default 'Y-m-d H:i:s'
 * @return DateTime $date
 */
function bwcs_start_date_in_site_timezone( $subscription, $format = 'Y-m-d H:i:s' )
{
	$synchronized = get_post_meta( $subscription->id, '_contains_synced_subscription', true ) === 'true';
	$trial_date = $subscription->get_date( 'trial_end' );
	
	if ( $synchronized || $trial_date ) {
		$date_type = 'next_payment';
	} else {
		$date_type = 'start';
	}
	
	$tz = get_option( 'timezone_string' );
	if ( $tz ) {
		// all subscription dates stored in the database are UTC
		$date = new DateTime( $subscription->get_date( $date_type ), new DateTimeZone( 'UTC' ) );
		$date->setTimezone( new DateTimeZone( $tz ) );
		$date = new DateTimeZone( $date->format( $format ), new DateTimeZone( 'UTC' ) );
	} else {
		$offset = get_option( 'gmt_offset' );
		$timestamp = $subscription->get_time( $date_type );
		$date_string = gmdate( $format, $timestamp + $offset * HOUR_IN_SECONDS );
		$date = DateTime::createFromFormat( $format, $date_string, new DateTimeZone( 'UTC' ) );
	}
	return $date;
}

/**
 *
 * @param WC_Subscription $subscription 
 * @return DateTime
 */
function bwcs_get_start_date_in_utc( $subscription )
{
	$synchronized = get_post_meta( $subscription->id, '_contains_synced_subscription', true ) === 'true';
	$trial_date = $subscription->get_date( 'trial_end' );
	
	if ( $synchronized || $trial_date ) {
		$date_type = 'next_payment';
	} else {
		$date_type = 'start';
	}
	// returns date in UTC.
	$start_date = $subscription->get_date( $date_type );
	return DateTime::createFromFormat( 'Y-m-d H:i:s', $start_date, new DateTimeZone( 'UTC' ) );
}

function bwcs_get_current_date_time()
{
	$tz = get_option( 'timezone_string' );
	if ( $tz ) {
		$date = new DateTimeZone( null, new DateTimeZone( $tz ) );
	} else {
		$offset = get_option( 'gmt_offset' );
		$date_string = gmdate( 'Y-m-d H:i:s', time() + $offset * HOUR_IN_SECONDS );
		$date = DateTime::createFromFormat( $format, $date_string, new DateTimeZone( 'UTC' ) );
	}
	return $date;
}

/**
 *
 * @param WC_Subscription $subscription 
 */
function bwcs_subscription_has_trial( $subscription )
{
	return ( bool ) $subscription->get_time( 'trial_end' );
}

/**
 *
 * @param WC_Subscription $subscription 
 */
function bwcs_subscription_is_synched( $subscription )
{
	return $subscription->contains_synced_subscription === 'true';
}

/**
 * Return true if the current request is for a payment method update on a failed renewal order.
 *
 * @return boolean
 */
function bwcs_is_paid_for_failed_renewal_request()
{
	return ( bool ) did_action( 'woocommerce_subscriptions_paid_for_failed_renewal_order' );
}